@@ -25,9 +25,11 @@
 #include <stdio.h>
 #include <glib.h>
 
+#include "genius-i18n.h"
+
 #include "structs.h"
 
-#define COPYRIGHT_STRING "Copyright (C) 1997-2003 George Lebl"
+#define COPYRIGHT_STRING "Copyright (C) 1997-2004 George Lebl"
 
 typedef enum {
 	NO_ERROR = 0,
@@ -126,6 +128,8 @@ void set_new_calcstate(calcstate_t state);
 void set_new_errorout(void (*func)(const char *));
 void set_new_infoout(void (*func)(const char *));
 
+void gel_errorout   (const char *format, ...) G_GNUC_PRINTF (1, 2);
+void gel_infoout   (const char *format, ...) G_GNUC_PRINTF (1, 2);
 /* this is bad */
 extern void (*errorout)(const char *);
 extern void (*infoout)(const char *);
@@ -197,4 +201,6 @@ extern char *gel_command_arg;
 
 extern GelOutput *main_out;
 
+extern gboolean gel_got_eof;
+
 #endif /* CALC_H_ */
